package com.amarinkovic.twitter.service.impl;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import twitter4j.Paging;
import twitter4j.Query;
import twitter4j.QueryResult;
import twitter4j.Status;
import twitter4j.Twitter;
import twitter4j.TwitterException;
import twitter4j.TwitterFactory;
import twitter4j.conf.ConfigurationBuilder;

import com.amarinkovic.twitter.model.Tweet;
import com.amarinkovic.twitter.service.ITwitterService;

/**
 * Service layer implementation for Twitter.
 * 
 * @author amarinkovic
 */
public class TwitterService implements ITwitterService {

	private Map<String, String> config;

	private static final String CONSUMER_KEY = "twitter4j.oauth.consumerKey";
	private static final String CONSUMER_SECRET = "twitter4j.oauth.consumerSecret";
	private static final String ACCESS_TOKEN = "twitter4j.oauth.accessToken";
	private static final String ACCESS_TOKEN_SECRET = "twitter4j.oauth.accessTokenSecret";
	
	private static final int DEFAULT_PAGE_SIZE = 50;
	private static final int DEFAULT_PAGE = 1;

	@Override
	public List<Tweet> getFriendsTimeline() throws TwitterException {
		Twitter twitter = getTweeter();
		List<Status> statuses = twitter.getFriendsTimeline(new Paging(DEFAULT_PAGE,	DEFAULT_PAGE_SIZE));
		List<Tweet> tweets = wrapStatuses(statuses);
		return tweets;
	}

	@Override
	public List<Tweet> getFriendsTimeline(int page, int count) throws TwitterException {
		Twitter twitter = getTweeter();
		List<Status> statuses = twitter.getFriendsTimeline(new Paging(page,	count));
		return wrapStatuses(statuses);
	}

	@Override
	public void updateStatus(String statusMessage) throws TwitterException {
		Twitter twitter = getTweeter();
		twitter.updateStatus(statusMessage);
	}

	@Override
	public List<Tweet> query(String qText) throws TwitterException {
		return query(qText, DEFAULT_PAGE, DEFAULT_PAGE_SIZE);
	}

	@Override
	public List<Tweet> query(String qText, int page, int count)	throws TwitterException {
		Twitter twitter = getTweeter();
		Query query = new Query(qText);
		query.setPage(page);
		query.setRpp(count);
		QueryResult qResult = twitter.search(query);
		List<twitter4j.Tweet> tweets = qResult.getTweets();
		return wrapTweets(tweets);
	}

	private Twitter getTweeter() {
		ConfigurationBuilder cb = new ConfigurationBuilder();
		cb.setDebugEnabled(true).setOAuthConsumerKey(config.get(CONSUMER_KEY))
				.setOAuthConsumerSecret(config.get(CONSUMER_SECRET))
				.setOAuthAccessToken(config.get(ACCESS_TOKEN))
				.setOAuthAccessTokenSecret(config.get(ACCESS_TOKEN_SECRET));
		TwitterFactory tf = new TwitterFactory(cb.build());
		Twitter twitter = tf.getInstance();
		return twitter;
	}

	private List<Tweet> wrapTweets(List<twitter4j.Tweet> rawTweets) {
		List<Tweet> tweets = new LinkedList<Tweet>();
		if (null != rawTweets) {
			for (twitter4j.Tweet rawTweet : rawTweets) {
				if (null != rawTweet) {
					tweets.add(wrapTweet(rawTweet));
				}
			}
		}
		return tweets;
	}

	private Tweet wrapTweet(twitter4j.Tweet rawTweet) {
		Tweet tweet = new Tweet();
		tweet.setCreatedAt(rawTweet.getCreatedAt());
		tweet.setId(rawTweet.getId());
		tweet.setSource(rawTweet.getSource());
		tweet.setText(rawTweet.getText());
		tweet.setFromUser(rawTweet.getFromUser());
		return tweet;
	}

	private List<Tweet> wrapStatuses(List<Status> statuses) {
		List<Tweet> tweets = new LinkedList<Tweet>();
		if (null != statuses) {
			for (Status status : statuses) {
				if (null != status) {
					tweets.add(wrapStatus(status));
				}
			}
		}
		return tweets;
	}

	private Tweet wrapStatus(Status status) {
		Tweet tweet = new Tweet();
		tweet.setCreatedAt(status.getCreatedAt());
		tweet.setId(status.getId());
		tweet.setSource(status.getSource());
		tweet.setText(status.getText());
		tweet.setFromUser(status.getUser().getName());
		return tweet;
	}

	public Map<String, String> getConfig() {
		return config;
	}

	public void setConfig(Map<String, String> config) {
		this.config = config;
	}
}